/********************************************************************************
* This file is used to program a bread maker home appliance developed as a part *
* of a thesis paper. The user is meant to change the MAC address variable to    *
* something that will not interfere with any other devices in his own network.  *
* This sketch is compatible with the Arduino IDE 1.6.X, although I strongly     *
* recommend using Codebender.cc.                                                *
*                                                                               *
* Student: Panagiotis Ioannidis (ionpan@gmail.com)                              *
* Supervisor: Minas Dasygenis (arch.icte.uowm.gr)                               *
* University of Western Macedonia, Faculty of Engineering                       *
* Department of Informatics & Telecommunications Engineering                    *
*********************************************************************************
* Copyright 2015 Panagiotis Ioannidis - Minas Dasygenis                         *
*                                                                               *
* Licensed under the Apache License, Version 2.0 (the "License");               *
* you may not use this file except in compliance with the License.              *
* You may obtain a copy of the License at                                       *
*                                                                               *
*     http://www.apache.org/licenses/LICENSE-2.0                                *
*                                                                               *
* Unless required by applicable law or agreed to in writing, software           *
* distributed under the License is distributed on an "AS IS" BASIS,             *
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.      *
* See the License for the specific language governing permissions and           *
* limitations under the License.                                                *
********************************************************************************/
#include <Nokia5110.h>
#include <SPI.h>
#include <Ethernet.h>

//#define SERIAL_DEBUG // if defined, enables serial feedback
#define NETWORK // if defined, enables network features

// thermistor values to degrees celsius
// 897 -> 100 degrees celcius
// 863 -> 110 degrees celcius
// 825 -> 120 degrees celcius
// 782 -> 130 degrees celcius
// 736 -> 140 degrees celcius
// 688 -> 150 degrees celcius

#define TEMP0 825 // 120 degrees
#define TEMP1 782 // 130 degrees
#define TEMP2 736 // 140 degrees
#define TEMP3 863 // 110 degrees

#ifdef NETWORK
uint8_t mac[] = {0x80, 0xA2, 0xDA, 0x0F, 0x19, 0x70}; // ethernet controller MAC address
char* const serverName[] PROGMEM = {"zafora.icte.uowm.gr"}; // remote server URL or IP
#define PORT 80 // remote server port to connect (default: 80)

// device credentials
unsigned long const device_id PROGMEM = 1;
char* const device_hash PROGMEM = {"a4c3152ca56bdbd3g94kab942acb6ba9"};
#endif

// peripherals pin assignments
#define ANALOG_BUTTONS A3
#define THERMISTOR A2
#define MOTOR A4
#define HEATER A5
#define POWER_BUTTON 7
#define LCD_LED 8
#define BUZZER 6

#define MAX_LCD_LED_TIME 20 // maximum time the LCD LED remains on (default: 20s)
#define MAX_USER_PRESENSE_INTERVAL 120 // maximum time counting after last user input (default: 120s)

#ifdef NETWORK
#define FETCH_INTERVAL 5 // time interval between two fetch retries (default: 5s)
#define RECONNECT_INTERVAL 300 // time interval between two reconnect retries (default: 300s)
#define MAX_FETCH_RETRIES 5 // maximum number of fetch tries before switching to reconnect (default: 5)
#endif

// LCD strings (due to memory limitations, all strings are stored in the local flash memory instead of RAM)
char* const strings_lcd_init[] PROGMEM = {"Programme: ", "Size: ", "Browning: ", "--:--:--", "Device Boot-Up"};
char* const strings_lcd_menu[] PROGMEM = {"1  ", "2  ", "3  ", "4  ", "5  ", "6  ", "7  ", "8  ", "9  ", "10 ", "11 ", "12 "};
char* const strings_lcd_loafsize[] PROGMEM = {"750gr   ", "900gr   ", "-       "};
char* const strings_lcd_browning[] PROGMEM = {"-   "};
char* const strings_lcd_time[] PROGMEM = {"   ", ":"};
#ifndef NETWORK
char* const strings_lcd_status[] PROGMEM = {"     Idle     ", "     Busy     ", "   Finished   ", "   Aborted    "}; // LCD status feedback (ethernet disabled)
char* const strings_lcd_net_status[] PROGMEM = {"              "}; // LCD network feedback (ethernet disabled)
#endif
#ifdef NETWORK
char* const strings_lcd_status[] PROGMEM = {"     Idle     ", "Manual Control", "   Finished   ", "   Aborted    ", "Remote Control"}; // LCD status feedback (ethernet enabled)
char* const strings_lcd_net_status[] PROGMEM = {"    Online    ", "   Offline    ", "  Connecting  ", "CredentialsErr", "Account Banned"}; // LCD network feedback (ethernet enabled)
#endif

// network related strings
#ifdef NETWORK
char* const ok_pattern[] PROGMEM = {"$$"}; // string pattern to find upon successful connect
char* const client_print_values[] PROGMEM = {"GET /~ictest00066/bread_online/commands.php?menu=", "&loafsize=", "&browning=", "&status=", "&countdown=", "&motor=", "&heater=", "&device_id=", "&device_hash=", "&status_change=", " HTTP/1.0"}; // URL parameters
#endif

// serial feedback strings (debugging purposes)
#ifdef SERIAL_DEBUG
char* const strings_serial_feedback_function[] PROGMEM = {"Menu: ", "Size: ", "Browning: ", "Max temperature: ", "Time remaining: ", "Device status: "};
char* const strings_serial_device_feedback[] PROGMEM = {"Initializing device...", "Device ready."};
#ifdef NETWORK
char* const serial_net_feedback[] PROGMEM = {"Initializing network connection...", "Error: Failed to configure Ethernet using DHCP", "Device IP:  ", "Subnet Mask: ", "Gateway IP:  ", "DNS IP: ", "Error: Could not connect to remote server.", "Reconnecting...", "Start command: ", "Stop command: ", "Remote Control", "Remotely aborted!", "Menu command: ", "Loafsize command: ", "Browning command: ", "Countdown command: ", "Invalid credentials", "Account banned"};
#endif
#endif

// device related variables
int menu, browning, loafsize, device_status = 0, temperature, buttons_value, even_odd_second, lcd_browning_counter; // buttons_value (read analog buttons value), even_odd_second (define whether a second is even or odd), lcd_browning_counter (counter for browning graphics)
// device_status: 0 (device Idle, ready), 1 (device busy, task initiated by manual user), 2 (device busy, task initiated by remote user), 3 (idle, task finished), 4 (idle, task aborted)
volatile int thermistor_value;
int seconds, ten_seconds, minutes, ten_minutes, hours, ten_hours; // LCD time parameters
unsigned long countdown, disp_timer, menu_min_time, menu_max_time = 359999; // countdown (time remaining for program), disp_timer (timer to edit and extract time parameters), menu_min_time (the minimum possible countdown for every program), menu_max_time (the maximum possible countdown for every program)
unsigned int abort_counter, user_presense_counter, buzzer_counter, heater_counter; // abort_counter (holding power button for 3 seconds aborts current program), user_presence_counter (time sice last user input), buzzer_counter (counts 3 seconds for buzzer)
char time_parts_char[2]; // convert time parameters to char for the LCD (necessary to display an integer on LCD)
bool buzzer_flag, buttons_flag = false, power_button_flag = false; // buzzer_flag (if raised, prevents buzzer from buzzing), buttons_flag (if raised, prevents debounce for browning, loafsize, menu, time+ and time- buttons), power_button_flag (if raised, prevents debounce for power button flag)

// network related variables
#ifdef NETWORK
int menu_command, browning_command, loafsize_command, start_command, stop_command; // commands read from remote server
unsigned long countdown_command; // command read from remote server
#ifdef NETWORK
int device_status_change = 0; // changes values if device_status changes, if operation initiates this changes to 1, if operation is finished this changes to 2, if operation is aborted by control panel this changes to 3, if operation is aborted remotely this changes to 3
bool device_status_change_flag = true; // allows the device_status_change variable to be reset to 0 when false
#endif
unsigned int fetch_retries_counter = 0; // number of unsuccessful fetch tries since last reconnection
bool connection_status; // 0 (connect is not available), 1 (connect is available)
volatile bool connect_flag = true, reconnect_flag = true; // connect_flag (give OK to connect), reconnect_flag (give OK to reconnect)
unsigned long connect_timer = 0, reconnect_timer = 0; // connect_timer (count time between connects), creonnect_timer (count time between reconnects)
int network_status; // 0 (offline), 1 (online, everything OK), 2 (connecting), 3 (invalid credentials)
#endif

#ifdef SERIAL_DEBUG
unsigned long serial_input;
#endif

#ifdef NETWORK
EthernetClient client; // create an ethernet client
#endif

Nokia5110 lcd(9, A1, 2, 5, 3); // create an lcd object

void motor_off_heater_off()
{
	digitalWrite(MOTOR, LOW);
	digitalWrite(HEATER, LOW);
}

void motor_on_heater_off()
{
	digitalWrite(MOTOR, HIGH);
	digitalWrite(HEATER, LOW);
}

// turn motor on/off every second
void motor_1sec_on_heater_off()
{
	// if second is even turn motor on, else turn motor off
	if (even_odd_second == 0)
	{
		digitalWrite(MOTOR, HIGH);
	}
	else if (even_odd_second == 1)
	{
		digitalWrite(MOTOR, LOW);
	}
	digitalWrite(HEATER, LOW);
}

void motor_off_heater_on()
{
	digitalWrite(MOTOR, LOW);
	// if thermistor value is lower than temperature limit + 35 (approaches target temperature), heater duty cycle = 1/3, else heater duty cycle = 1
	if (thermistor_value <= temperature + 35)
	{
		if ((thermistor_value > temperature + 5) && (heater_counter >= 20) && (heater_counter < 40))
		{
			digitalWrite(HEATER, HIGH);
		}
		else
		{
			digitalWrite(HEATER, LOW);
		}
	}
	else
	{
		digitalWrite(HEATER, HIGH);
	}
}

void motor_on_heater_on()
{
	digitalWrite(MOTOR, HIGH);
	// if thermistor value is lower than temperature limit + 35 (approaches target temperature), heater duty cycle = 1/3, else heater duty cycle = 1
	if (thermistor_value <= temperature + 35)
	{
		if ((thermistor_value > temperature + 5) && (heater_counter >= 20) && (heater_counter < 40))
		{
			digitalWrite(HEATER, HIGH);
		}
		else
		{
			digitalWrite(HEATER, LOW);
		}
	}
	else
	{
		digitalWrite(HEATER, HIGH);
	}
}

void menu1_loafsize0_routine()
{
	if (countdown > 10380)
	{
		motor_off_heater_off();
	}
	else if (countdown > 10260)
	{
		motor_1sec_on_heater_off();
	}
	else if (countdown > 10080)
	{
		if (countdown == 10231 || countdown == 10201 || countdown == 10171 || countdown == 10141 || countdown == 10111 || countdown == 10081)
		{
			motor_off_heater_off();
		}
		else
		{
			motor_on_heater_off();
		}
	}
	else if (countdown > 9840)
	{
		motor_on_heater_off();
	}
	else if (countdown > 8640)
	{
		if (countdown == 9840)
		{
			buzzer_flag = false;
		}
		motor_off_heater_off();
	}
	else if (countdown > 7800)
	{
		motor_on_heater_off();
	}
	else if (countdown > 6600)
	{
		motor_off_heater_off();
	}
	else if (countdown > 6570)
	{
		motor_on_heater_off();
	}
	else if (countdown > 3600)
	{
		motor_off_heater_off();
	}
	else if (countdown > 0)
	{
		motor_off_heater_on();
	}
}

void menu1_loafsize1_routine()
{
	if (countdown > 10800)
	{
		motor_off_heater_off();
	}
	else if (countdown > 10680)
	{
		motor_1sec_on_heater_off();
	}
	else if (countdown > 10500)
	{
		if (countdown == 10651 || countdown == 10621 || countdown == 10591 || countdown == 10561 || countdown == 10531 || countdown == 10501)
		{
			motor_off_heater_off();
		}
		else
		{
			motor_on_heater_off();
		}
	}
	else if (countdown > 10200)
	{
		motor_on_heater_off();
	}
	else if (countdown > 9000)
	{
		if (countdown == 10200)
		{
			buzzer_flag = false;
		}
		motor_off_heater_off();
	}
	else if (countdown > 8100)
	{
		motor_on_heater_off();
	}
	else if (countdown > 6900)
	{
		motor_off_heater_off();
	}
	else if (countdown > 6870)
	{
		motor_on_heater_off();
	}
	else if (countdown > 6600)
	{
		motor_off_heater_off();
	}
	else if (countdown > 3900)
	{
		motor_off_heater_off();
	}
	else if (countdown > 0)
	{
		motor_off_heater_on();
	}
}

void menu2_loafsize0_routine()
{
	if (countdown > 13200)
	{
		motor_off_heater_off();
	}
	else if (countdown > 13080)
	{
		motor_1sec_on_heater_off();
	}
	else if (countdown > 12900)
	{
		if (countdown == 13051 || countdown == 13021 || countdown == 12991 || countdown == 12961 || countdown == 12931 || countdown == 12901)
		{
			motor_off_heater_off();
		}
		else
		{
			motor_on_heater_off();
		}
	}
	else if (countdown > 12240)
	{
		motor_on_heater_off();
	}
	else if (countdown > 9840)
	{
		if (countdown == 12240)
		{
			buzzer_flag = false;
		}
		motor_off_heater_off();
	}
	else if (countdown > 8700)
	{
		motor_on_heater_off();
	}
	else if (countdown > 7500)
	{
		motor_off_heater_off();
	}
	else if (countdown > 7470)
	{
		motor_on_heater_off();
	}
	else if (countdown > 3900)
	{
		motor_off_heater_off();
	}
	else if (countdown > 0)
	{
		motor_off_heater_on();
	}
}

void menu2_loafsize1_routine()
{
	if (countdown > 13800)
	{
		motor_off_heater_off();
	}
	else if (countdown > 13680)
	{
		motor_1sec_on_heater_off();
	}
	else if (countdown > 13500)
	{
		if (countdown == 13651 || countdown == 13621 || countdown == 13591 || countdown == 13561 || countdown == 13531 || countdown == 13501)
		{
			motor_off_heater_off();
		}
		else
		{
			motor_on_heater_off();
		}
	}
	else if (countdown > 12720)
	{
		motor_on_heater_off();
	}
	else if (countdown > 10320)
	{
		if (countdown == 12720)
		{
			buzzer_flag = false;
		}
		motor_off_heater_off();
	}
	else if (countdown > 9000)
	{
		motor_on_heater_off();
	}
	else if (countdown > 7800)
	{
		motor_off_heater_off();
	}
	else if (countdown > 7770)
	{
		motor_on_heater_off();
	}
	else if (countdown > 4200)
	{
		motor_off_heater_off();
	}
	else if (countdown > 0)
	{
		motor_off_heater_on();
	}
}

void menu3_loafsize0_routine()
{
	if (countdown > 12720)
	{
		motor_off_heater_off();
	}
	else if (countdown > 12600)
	{
		motor_1sec_on_heater_off();
	}
	else if (countdown > 12420)
	{
		if (countdown == 12571 || countdown == 12541 || countdown == 12511 || countdown == 12481 || countdown == 12451 || countdown == 12421)
		{
			motor_off_heater_off();
		}
		else
		{
			motor_on_heater_off();
		}
	}
	else if (countdown > 12180)
	{
		motor_on_heater_off();
	}
	else if (countdown > 10680)
	{
		motor_off_heater_off();
	}
	else if (countdown > 9600)
	{
		if (countdown == 10680)
		{
			buzzer_flag = false;
		}
		motor_on_heater_off();
	}
	else if (countdown > 8400)
	{
		motor_off_heater_off();
	}
	else if (countdown > 8370)
	{
		motor_on_heater_off();
	}
	else if (countdown > 3300)
	{
		motor_off_heater_off();
	}
	else if (countdown > 0)
	{
		motor_off_heater_on();
	}
}

void menu3_loafsize1_routine()
{
	if (countdown > 13200)
	{
		motor_off_heater_off();
	}
	else if (countdown > 13080)
	{
		motor_1sec_on_heater_off();
	}
	else if (countdown > 12900)
	{
		if (countdown == 13051 || countdown == 13021 || countdown == 12991 || countdown == 12961 || countdown == 12931 || countdown == 12901)
		{
			motor_off_heater_off();
		}
		else
		{
			motor_on_heater_off();
		}
	}
	else if (countdown > 12600)
	{
		motor_on_heater_off();
	}
	else if (countdown > 11100)
	{
		motor_off_heater_off();
	}
	else if (countdown > 9900)
	{
		if (countdown == 11100)
		{
			buzzer_flag = false;
		}
		motor_on_heater_off();
	}
	else if (countdown > 8700)
	{
		motor_off_heater_off();
	}
	else if (countdown > 8670)
	{
		motor_on_heater_off();
	}
	else if (countdown > 3600)
	{
		motor_off_heater_off();
	}
	else if (countdown > 0)
	{
		motor_off_heater_on();
	}
}

void menu4_routine()
{
	if (countdown > 6000)
	{
		motor_off_heater_off();
	}
	else if (countdown > 5880)
	{
		motor_1sec_on_heater_off();
	}
	else if (countdown > 5700)
	{
		if (countdown == 5851 || countdown == 5821 || countdown == 5791 || countdown == 5761 || countdown == 5731 || countdown == 5701)
		{
			motor_off_heater_off();
		}
		else
		{
			motor_on_heater_off();
		}
	}
	else if (countdown > 5580)
	{
		motor_on_heater_off();
	}
	else if (countdown > 5280)
	{
		motor_off_heater_off();
	}
	else if (countdown > 4800)
	{
		motor_on_heater_off();
	}
	else if (countdown > 0)
	{
		motor_off_heater_on();
	}
}

void menu5_loafsize0_routine()
{
	if (countdown > 10200)
	{
		motor_off_heater_off();
	}
	else if (countdown > 10080)
	{
		motor_1sec_on_heater_off();
	}
	else if (countdown > 9990)
	{
		if (countdown == 10051 || countdown == 10021 || countdown == 9991 || countdown == 9961 || countdown == 9931 || countdown == 9901)
		{
			motor_off_heater_off();
		}
		else
		{
			motor_on_heater_off();
		}
	}
	else if (countdown > 9600)
	{
		motor_on_heater_off();
	}
	else if (countdown > 9300)
	{
		motor_off_heater_off();
	}
	else if (countdown > 8100)
	{
		if (countdown == 9300)
		{
			buzzer_flag = false;
		}
		motor_on_heater_off();
	}
	else if (countdown > 6900)
	{
		motor_off_heater_off();
	}
	else if (countdown > 6870)
	{
		motor_on_heater_off();
	}
	else if (countdown > 3000)
	{
		motor_off_heater_off();
	}
	else if (countdown > 0)
	{
		motor_off_heater_on();
	}
}

void menu5_loafsize1_routine()
{
	if (countdown > 10500)
	{
		motor_off_heater_off();
	}
	else if (countdown > 10380)
	{
		motor_1sec_on_heater_off();
	}
	else if (countdown > 10200)
	{
		if (countdown == 10351 || countdown == 10321 || countdown == 10291 || countdown == 10261 || countdown == 10231 || countdown == 10201)
		{
			motor_off_heater_off();
		}
		else
		{
			motor_on_heater_off();
		}
	}
	else if (countdown > 9900)
	{
		motor_on_heater_off();
	}
	else if (countdown > 9600)
	{
		motor_off_heater_off();
	}
	else if (countdown > 8400)
	{
		if (countdown == 9600)
		{
			buzzer_flag = false;
		}
		motor_on_heater_off();
	}
	else if (countdown > 7200)
	{
		motor_off_heater_off();
	}
	else if (countdown > 7770)
	{
		motor_on_heater_off();
	}
	else if (countdown > 3600)
	{
		motor_off_heater_off();
	}
	else if (countdown > 0)
	{
		motor_off_heater_on();
	}
}

void menu6_routine()
{
	if (countdown > 3480)
	{
		motor_off_heater_off();
	}
	else if (countdown > 3360)
	{
		motor_1sec_on_heater_off();
	}
	else if (countdown > 3180)
	{
		if (countdown == 3331 || countdown == 3301 || countdown == 3271 || countdown == 3241 || countdown == 3211 || countdown == 3181)
		{
			motor_off_heater_off();
		}
		else
		{
			motor_on_heater_off();
		}
	}
	else if (countdown > 2760)
	{
		motor_on_heater_off();
	}
	else if (countdown > 2100)
	{
		motor_off_heater_off();
	}
	else if (countdown > 0)
	{
		motor_off_heater_on();
	}
}

void menu7_routine()
{
	if (countdown > 3480)
	{
		motor_off_heater_off();
	}
	else if (countdown > 3360)
	{
		motor_1sec_on_heater_off();
	}
	else if (countdown > 3180)
	{
		if (countdown == 3331 || countdown == 3301 || countdown == 3271 || countdown == 3241 || countdown == 3211 || countdown == 3181)
		{
			motor_off_heater_off();
		}
		else
		{
			motor_on_heater_off();
		}
	}
	else if (countdown > 3000)
	{
		motor_on_heater_off();
	}
	else if (countdown > 2400)
	{
		motor_off_heater_off();
	}
	else if (countdown > 0)
	{
		motor_off_heater_on();
	}
}

void menu8_routine()
{
	if (countdown > 5400)
	{
		motor_off_heater_off();
	}
	else if (countdown > 5280)
	{
		motor_1sec_on_heater_off();
	}
	else if (countdown > 5100)
	{
		if (countdown == 5251 || countdown == 5221 || countdown == 5191 || countdown == 5161 || countdown == 5131 || countdown == 5101)
		{
			motor_off_heater_off();
		}
		else
		{
			motor_on_heater_off();
		}
	}
	else if (countdown > 4200)
	{
		motor_on_heater_off();
	}
	else if (countdown > 3000)
	{
		motor_off_heater_off();
	}
	else if (countdown > 2970)
	{
		motor_on_heater_off();
	}
	else if (countdown > 0)
	{
		motor_off_heater_off();
	}
}

void menu9_routine()
{
	if (countdown > 4800)
	{
		motor_off_heater_off();
	}
	else if (countdown > 3900)
	{
		motor_off_heater_on();
	}
	else if (countdown > 1200)
	{
		motor_on_heater_on();
	}
	else if (countdown > 0)
	{
		motor_off_heater_off();
	}
}

void menu10_routine()
{
	if (countdown > 10200)
	{
		motor_off_heater_off();
	}
	else if (countdown > 10080)
	{
		motor_1sec_on_heater_off();
	}
	else if (countdown > 9990)
	{
		if (countdown == 10051 || countdown == 10021 || countdown == 9991 || countdown == 9961 || countdown == 9931 || countdown == 9901)
		{
			motor_off_heater_off();
		}
		else
		{
			motor_on_heater_off();
		}
	}
	else if (countdown > 9600)
	{
		motor_on_heater_off();
	}
	else if (countdown > 9300)
	{
		motor_off_heater_off();
	}
	else if (countdown > 8100)
	{
		if (countdown == 9300)
		{
			buzzer_flag = false;
		}
		motor_on_heater_off();
	}
	else if (countdown > 6360)
	{
		motor_off_heater_off();
	}
	else if (countdown > 6330)
	{
		motor_on_heater_off();
	}
	else if (countdown > 4200)
	{
		motor_off_heater_off();
	}
	else if (countdown > 0)
	{
		motor_off_heater_on();
	}
}

void menu11_loafsize0_routine()
{
	if (countdown > 10500)
	{
		motor_off_heater_off();
	}
	else if (countdown > 10380)
	{
		motor_1sec_on_heater_off();
	}
	else if (countdown > 10200)
	{
		if (countdown == 10351 || countdown == 10321 || countdown == 10291 || countdown == 10261 || countdown == 10231 || countdown == 10201)
		{
			motor_off_heater_off();
		}
		else
		{
			motor_on_heater_off();
		}
	}
	else if (countdown > 9600)
	{
		motor_on_heater_off();
	}
	else if (countdown > 7200)
	{
		motor_off_heater_off();
	}
	else if (countdown > 6900)
	{
		if (countdown == 7200)
		{
			buzzer_flag = false;
		}
		motor_on_heater_off();
	}
	else if (countdown > 5460)
	{
		motor_off_heater_off();
	}
	else if (countdown > 5430)
	{
		motor_on_heater_off();
	}
	else if (countdown > 3000)
	{
		motor_off_heater_off();
	}
	else if (countdown > 0)
	{
		motor_off_heater_on();
	}
}

void menu11_loafsize1_routine()
{
	if (countdown > 10800)
	{
		motor_off_heater_off();
	}
	else if (countdown > 10680)
	{
		motor_1sec_on_heater_off();
	}
	else if (countdown > 10500)
	{
		if (countdown == 10651 || countdown == 10621 || countdown == 10591 || countdown == 10561 || countdown == 10531 || countdown == 10501)
		{
			motor_off_heater_off();
		}
		else
		{
			motor_on_heater_off();
		}
	}
	else if (countdown > 9900)
	{
		motor_on_heater_off();
	}
	else if (countdown > 7500)
	{
		motor_off_heater_off();
	}
	else if (countdown > 7200)
	{
		if (countdown == 7500)
		{
			buzzer_flag = false;
		}
		motor_on_heater_off();
	}
	else if (countdown > 5760)
	{
		motor_off_heater_off();
	}
	else if (countdown > 5730)
	{
		motor_on_heater_off();
	}
	else if (countdown > 3300)
	{
		motor_off_heater_off();
	}
	else if (countdown > 0)
	{
		motor_off_heater_on();
	}
}

void menu12_routine()
{
	if (countdown > 3600)
	{
		motor_off_heater_off();
	}
	else if (countdown > 0)
	{
		motor_off_heater_on();
	}
}

// serial feedback for debugging puproses
#ifdef SERIAL_DEBUG
void serial_feedback()
{
	// info for selected menu
	Serial.print(strings_serial_feedback_function[0]);
	Serial.println(menu);
	// info for selected loafsize
	Serial.print(strings_serial_feedback_function[1]);
	Serial.println(loafsize);
	// info for selected browning
	Serial.print(strings_serial_feedback_function[2]);
	Serial.println(browning);
	// info for selected temperature (based on browning selection)
	Serial.print(strings_serial_feedback_function[3]);
	Serial.println(temperature);
	// info for time remaining
	Serial.print(strings_serial_feedback_function[4]);
	Serial.println(countdown);
	// info for device status
	Serial.print(strings_serial_feedback_function[5]);
	Serial.println(device_status);
	Serial.println();
}
#endif

// set temperature for browning selection
void browning_temp()
{
	if (browning == 0)
	{
		temperature = TEMP0;
	}
	else if (browning == 1)
	{
		temperature = TEMP1;
	}
	else if (browning == 2)
	{
		temperature = TEMP2;
	}
	if (menu == 9)
	{
		temperature = TEMP3;
	}
	if (menu == 11 || menu == 12)
	{
		temperature = TEMP0;
	}
	else if (menu == 10)
	{
		temperature = TEMP2;
	}
}

// set LCD messages for menu selection
void lcd_menu()
{
	lcd.gotoXY(0, 0);
	lcd.string(strings_lcd_init[0]);
	if (menu == 1)
	{
		lcd.string(strings_lcd_menu[0]);
	}
	else if (menu == 2)
	{
		lcd.string(strings_lcd_menu[1]);
	}
	else if (menu == 3)
	{
		lcd.string(strings_lcd_menu[2]);
	}
	else if (menu == 4)
	{
		lcd.string(strings_lcd_menu[3]);
	}
	else if (menu == 5)
	{
		lcd.string(strings_lcd_menu[4]);
	}
	else if (menu == 6)
	{
		lcd.string(strings_lcd_menu[5]);
	}
	else if (menu == 7)
	{
		lcd.string(strings_lcd_menu[6]);
	}
	else if (menu == 8)
	{
		lcd.string(strings_lcd_menu[7]);
	}
	else if (menu == 9)
	{
		lcd.string(strings_lcd_menu[8]);
	}
	else if (menu == 10)
	{
		lcd.string(strings_lcd_menu[9]);
	}
	else if (menu == 11)
	{
		lcd.string(strings_lcd_menu[10]);
	}
	else if (menu == 12)
	{
		lcd.string(strings_lcd_menu[11]);
	}
}

// set LCD messages for loafsize selection
void lcd_loafsize()
{
	lcd.gotoXY(0, 1);
	lcd.string(strings_lcd_init[1]);
	if (loafsize == 0)
	{
		lcd.string(strings_lcd_loafsize[0]);
	}
	else if (loafsize == 1)
	{
		lcd.string(strings_lcd_loafsize[1]);
	}
	else if (loafsize == 2)
	{
		lcd.string(strings_lcd_loafsize[2]);
	}
}

// set LCD messages for browning selection
void lcd_browning()
{
	lcd.gotoXY(0, 2);
	lcd.string(strings_lcd_init[2]);
	lcd.gotoXY(59, 2);
	if (browning == 0)
	{
		lcd.write(LCD_DATA, 0x7f);
		for (lcd_browning_counter = 0; lcd_browning_counter < 23; lcd_browning_counter++)
		{
			lcd.write(LCD_DATA, 0x41);
		}
		lcd.write(LCD_DATA, 0x7f);
	}
	else if (browning == 1)
	{
		lcd.write(LCD_DATA, 0x7f);
		for (lcd_browning_counter = 0; lcd_browning_counter < 12; lcd_browning_counter++)
		{
			lcd.write(LCD_DATA, 0x41);
			lcd.write(LCD_DATA, 0x7f);
		}
	}
	else if (browning == 2)
	{
		for (lcd_browning_counter = 0; lcd_browning_counter < 25; lcd_browning_counter++)
		{
			lcd.write(LCD_DATA, 0x7f);
		}
	}
	else if (browning == 3)
	{
		lcd.string(strings_lcd_browning[0]);
		lcd.write(LCD_DATA, 0x00);
	}
}

// set LCD messages for network status
#ifdef NETWORK
void lcd_network_status()
{
	lcd.gotoXY(0, 4);
	if (network_status == 0)
	{
		lcd.string(strings_lcd_net_status[1]);
	}
	else if (network_status == 1)
	{
		lcd.string(strings_lcd_net_status[0]);
	}
	else if (network_status == 2)
	{
		lcd.string(strings_lcd_net_status[2]);
	}
	else if (network_status == 3)
	{
		lcd.string(strings_lcd_net_status[3]);
	}
	else if (network_status == 4)
	{
		lcd.string(strings_lcd_net_status[4]);
	}
}
#endif

// set LCD messages for device status
void lcd_status()
{
	lcd.gotoXY(0, 5);
	if (device_status == 0)
	{
		lcd.string(strings_lcd_status[0]);
	}
	else if (device_status == 1)
	{
		lcd.string(strings_lcd_status[1]);
	}
	else if (device_status == 3)
	{
		lcd.string(strings_lcd_status[2]);
	}
	else if (device_status == 4)
	{
		lcd.string(strings_lcd_status[3]);
	}
#ifdef NETWORK
	else if (device_status == 2)
	{
		lcd.string(strings_lcd_status[4]);
	}
#endif
}

// set LCD messages for time
void lcd_time()
{
	disp_timer = countdown;

	// calculate digits for every time parameter
	hours = disp_timer / 3600;
	ten_hours = hours / 10;
	hours = hours % 10;
	disp_timer = disp_timer % 3600;
	minutes = disp_timer / 60;
	ten_minutes = minutes / 10;
	minutes = minutes % 10;
	seconds = disp_timer % 60;
	ten_seconds = seconds / 10;
	seconds = seconds % 10;

	// check if second is even of odd
	if (seconds == 0 || seconds == 2 || seconds == 4 || seconds == 6 || seconds == 8)
	{
		even_odd_second = 0; // even second
	}
	else if (seconds == 1 || seconds == 3 || seconds == 5 || seconds == 7 || seconds == 9)
	{
		even_odd_second = 1; // odd second
	}

	// display digits for every time parameter
	lcd.gotoXY(0, 3);
	lcd.string(strings_lcd_time[0]);
	itoa(ten_hours, time_parts_char, 10);
	lcd.string(time_parts_char);
	itoa(hours, time_parts_char, 10);
	lcd.string(time_parts_char);
	lcd.string(strings_lcd_time[1]);
	itoa(ten_minutes, time_parts_char, 10);
	lcd.string(time_parts_char);
	itoa(minutes, time_parts_char, 10);
	lcd.string(time_parts_char);
	lcd.string(strings_lcd_time[1]);
	itoa(ten_seconds, time_parts_char, 10);
	lcd.string(time_parts_char);
	itoa(seconds, time_parts_char, 10);
	lcd.string(time_parts_char);
	lcd.string(strings_lcd_time[0]);
#ifndef NETWORK
	lcd.string(strings_lcd_net_status[0]);
#endif
}

// refresh LCD info
void display_routine()
{
	lcd_menu();
	lcd_loafsize();
	lcd_browning();
	lcd_time();
#ifdef NETWORK
	lcd_network_status();
#endif
	lcd_status();
}

// initialize device on default values
void device_init()
{
	if (device_status != 3 && device_status != 4)
	{
		device_status = 0;
	}
	menu = 1;
	loafsize = 0;
	browning = 0;
	countdown = 10380;
	browning_temp();
	menu_min_time = countdown;
	motor_off_heater_off();
	digitalWrite(LCD_LED, LOW);
	user_presense_counter = 0;
	abort_counter = 0;
	heater_counter = 0;
	buzzer_flag = true;
	buzzer_counter = 0;
	lcd.init();
	lcd.clear();
	display_routine();
}

// select loafsize value
void loafsize_routine()
{
	if (menu == 1)
	{
		if (loafsize == 0)
		{
			countdown = 10380;
		}
		else if (loafsize == 1)
		{
			countdown = 10800;
		}
	}
	else if (menu == 2)
	{
		if (loafsize == 0)
		{
			countdown = 13200;
		}
		else if (loafsize == 1)
		{
			countdown = 13800;
		}
	}
	else if (menu == 3)
	{
		if (loafsize == 0)
		{
			countdown = 12720;
		}
		else if (loafsize == 1)
		{
			countdown = 13200;
		}
	}
	else if (menu == 5)
	{
		if (loafsize == 0)
		{
			countdown = 10200;
		}
		else if (loafsize == 1)
		{
			countdown = 10500;
		}
	}
	else if (menu == 11)
	{
		if (loafsize == 0)
		{
			countdown = 10500;
		}
		else if (loafsize == 1)
		{
			countdown = 10800;
		}
	}
}

// select menu value and set default values for loafsize, browning and countdown
void menu_routine()
{
	if (menu == 1)
	{
		loafsize = 0;
		browning = 0;
		countdown = 10380;
	}
	else if (menu == 2)
	{
		loafsize = 0;
		browning = 0;
		countdown = 13200;
	}
	else if (menu == 3)
	{
		loafsize = 0;
		browning = 0;
		countdown = 12720;
	}
	else if (menu == 4)
	{
		loafsize = 2;
		browning = 0;
		countdown = 6000;
	}
	else if (menu == 5)
	{
		loafsize = 0;
		browning = 0;
		countdown = 10200;
	}
	else if (menu == 6)
	{
		loafsize = 0;
		browning = 0;
		countdown = 3480;
	}
	else if (menu == 7)
	{
		loafsize = 1;
		browning = 0;
		countdown = 3480;
	}
	else if (menu == 8)
	{
		loafsize = 1;
		browning = 3;
		countdown = 5400;
	}
	else if (menu == 9)
	{
		loafsize = 2;
		browning = 3;
		countdown = 4800;
	}
	else if (menu == 10)
	{
		loafsize = 2;
		browning = 3;
		countdown = 10200;
	}
	else if (menu == 11)
	{
		loafsize = 0;
		browning = 3;
		countdown = 10500;
	}
	else if (menu == 12)
	{
		loafsize = 2;
		browning = 3;
		countdown = 3600;
	}
}

// browning, loafsize, menu, time+ and time- buttons input
void analog_buttons()
{
	buttons_value = analogRead(ANALOG_BUTTONS);

	// browning button
	if (buttons_value >= 800 && buttons_value <= 950 && buttons_flag == false && menu >= 1 && menu <= 7)
	{
		buttons_flag = true;
		if (user_presense_counter <= MAX_LCD_LED_TIME) // if LCD LED is on, continue
		{
			user_presense_counter = 0;
			if (device_status == 0)
			{
				browning++;
				if (browning == 3)
				{
					browning = 0;
				}
				browning_temp();
				display_routine();
#ifdef SERIAL_DEBUG
				serial_feedback();
#endif
			}
		}
		// if LCD LED is off, turn it on and do nothing else
		else
		{
			user_presense_counter = 0;
			digitalWrite(LCD_LED, LOW);
		}
	}
	// loafsize button
	else if (buttons_value >= 680 && buttons_value <= 780 && buttons_flag == false && (menu == 1 || menu == 2 || menu == 3 || menu == 5 || menu == 11))
	{
		buttons_flag = true;
		if (user_presense_counter <= MAX_LCD_LED_TIME) // if LCD LED is on, continue
		{
			user_presense_counter = 0;
			if (device_status == 0)
			{
				loafsize++;
				if (loafsize == 2)
				{
					loafsize = 0;
				}
				loafsize_routine();
				display_routine();
				menu_min_time = countdown;
#ifdef SERIAL_DEBUG
				serial_feedback();
#endif
			}
		}
		// if LCD LED is off, turn it on and do nothing else
		else
		{
			user_presense_counter = 0;
			digitalWrite(LCD_LED, LOW);
		}
	}
	// menu button
	else if (buttons_value >= 380 && buttons_value <= 500 && buttons_flag == false)
	{
		buttons_flag = true;
		if (user_presense_counter <= MAX_LCD_LED_TIME) // if LCD LED is on, continue
		{
			user_presense_counter = 0;
			if (device_status == 0)
			{
				menu++;
				if (menu == 13)
				{
					menu = 1;
				}
				menu_routine();
				browning_temp();
				display_routine();
				if (menu != 12)
				{
					menu_min_time = countdown;
				}
				else
				{
					menu_min_time = 600;
				}
#ifdef SERIAL_DEBUG
				serial_feedback();
#endif
			}
		}
		// if LCD LED is off, turn it on and do nothing else
		else
		{
			user_presense_counter = 0;
			digitalWrite(LCD_LED, LOW);
		}
	}
	// timer+ button
	else if (buttons_value >= 250 && buttons_value <= 350 && buttons_flag == false)
	{
		buttons_flag = true;
		if (user_presense_counter <= MAX_LCD_LED_TIME) // if LCD LED is on, continue
		{
			user_presense_counter = 0;
			if (device_status == 0)
			{
				if (countdown + 600 < menu_max_time)
				{
					countdown += 600; // if countdown + 10 minutes is less than 99:59:59, add 10 minutes to countdown
				}
				display_routine();
#ifdef SERIAL_DEBUG
				serial_feedback();
#endif
			}
		}
		// if LCD LED is off, turn it on and do nothing else
		else
		{
			user_presense_counter = 0;
			digitalWrite(LCD_LED, LOW);
		}
	}
	// timer- button
	else if (buttons_value >= 100 && buttons_value <= 200 && buttons_flag == false)
	{
		buttons_flag = true;
		if (user_presense_counter <= MAX_LCD_LED_TIME) // if LCD LED is on, continue
		{
			user_presense_counter = 0;
			if (device_status == 0)
			{
				if ((countdown - 600) >= menu_min_time)
				{
					countdown -= 600; // if countdown - 10 minutes is equal to the least possible countdown (based on menu duration), abstract 10 minutes from countdown
				}
				display_routine();
#ifdef SERIAL_DEBUG
				serial_feedback();
#endif
			}
		}
		// if LCD LED is off, turn it on and do nothing else
		else
		{
			user_presense_counter = 0;
			digitalWrite(LCD_LED, LOW);
		}
	}
	// if no button is pressed, allow new button input
	else if (buttons_value < 50 && buttons_flag == true)
	{
		buttons_flag = false;
	}
}

// power button input
void power_button()
{
	if (power_button_flag == false && digitalRead(POWER_BUTTON) == LOW)
	{
		power_button_flag = true;
		if (user_presense_counter <= MAX_LCD_LED_TIME) // if LCD LED is on, continue
		{
			user_presense_counter = 0;
			if (device_status == 0)
			{
				device_status = 1; // turn device status to manual control
#ifdef NETWORK
				device_status_change = 1;
#endif
				buzzer_flag = false; // allow buzzer to buzz
			}
			else if (device_status == 3 || device_status == 4) // if finished or aborted, become available
			{
				device_status = 0;
			}
			display_routine();
#ifdef SERIAL_DEBUG
			serial_feedback();
#endif
		}
		// if LCD LED is off, turn it on and do nothing else
		else
		{
			user_presense_counter = 0;
			digitalWrite(LCD_LED, LOW);
		}
	}
	// if power button is not pressed, allow new input
	else if (digitalRead(POWER_BUTTON) == HIGH)
	{
		power_button_flag = false;
	}

}

// set new countdown value (for debugging purposes)
#ifdef SERIAL_DEBUG
void serialEvent()
{
	serial_input = 0;
	while (Serial.available() > 0)
	{
		serial_input *= 10;
		serial_input += (Serial.read() - '0');
		delay(5);
	}
	countdown = serial_input;
}
#endif

// buttons input while device is busy
void runtime_buttons()
{
	buttons_value = analogRead(ANALOG_BUTTONS);
	// if any button (except power button) is pressed
	if ((buttons_value >= 800 && buttons_value <= 950) || (buttons_value >= 680 && buttons_value <= 780) || digitalRead(POWER_BUTTON) == LOW || (buttons_value >= 380 && buttons_value <= 500) || (buttons_value >= 250 && buttons_value <= 350) || (buttons_value >= 100 && buttons_value <= 200))
	{
		if (user_presense_counter <= MAX_LCD_LED_TIME)
		{
			user_presense_counter = 0;
		}
		else
		{
			user_presense_counter = 0;
			digitalWrite(LCD_LED, LOW);
		}
	}
}

void runtime_routines()
{
	thermistor_value = analogRead(THERMISTOR); // read thermistor value

	if (menu == 1)
	{
		if (loafsize == 0)
		{
			menu1_loafsize0_routine();
		}
		else if (loafsize == 1)
		{
			menu1_loafsize1_routine();
		}
	}
	else if (menu == 2)
	{
		if (loafsize == 0)
		{
			menu2_loafsize0_routine();
		}
		else if (loafsize == 1)
		{
			menu2_loafsize1_routine();
		}
	}
	else if (menu == 3)
	{
		if (loafsize == 0)
		{
			menu3_loafsize0_routine();
		}
		else if (loafsize == 1)
		{
			menu3_loafsize1_routine();
		}
	}
	else if (menu == 4)
	{
		menu4_routine();
	}
	else if (menu == 5)
	{
		if (loafsize == 0)
		{
			menu5_loafsize0_routine();
		}
		else if (loafsize == 1)
		{
			menu5_loafsize1_routine();
		}
	}
	else if (menu == 6)
	{
		menu6_routine();
	}
	else if (menu == 7)
	{
		menu7_routine();
	}
	else if (menu == 8)
	{
		menu8_routine();
	}
	else if (menu == 9)
	{
		menu9_routine();
	}
	else if (menu == 10)
	{
		menu10_routine();
	}
	else if (menu == 11)
	{
		if (loafsize == 0)
		{
			menu11_loafsize0_routine();
		}
		else if (loafsize == 1)
		{
			menu11_loafsize1_routine();
		}
	}
	else if (menu == 12)
	{
		menu12_routine();
	}
}

// run every second
ISR(TIMER1_COMPA_vect)
{
#ifdef NETWORK
	// if connection_status == true and credentials are valid or account not banned, count seconds to connect, else, count seconds to reconnect
	if (connection_status == true && (network_status != 3 || network_status != 4))
	{
		connect_timer++;
		// if fetch interval is reached drop flag to connect
		if (connect_timer >= FETCH_INTERVAL)
		{
			connect_flag = false;
			connect_timer = 0;
		}
	}
	else if (network_status != 3 || network_status != 4)
	{
		reconnect_timer++;
		// if reconnect interval is reached, drop flag to reconnect
		if (reconnect_timer >= RECONNECT_INTERVAL)
		{
			reconnect_flag = false;
			reconnect_timer = 0;
		}
	}
#endif

	// if buzzer flag is dropped, enable buzz for 3 seconds
	if (buzzer_flag == false)
	{
		analogWrite(BUZZER, 200);
		buzzer_counter++;
	}
	// if buzzer is on for 3 seconds, turn it off and raise buzzer flag
	if (buzzer_counter >= 3)
	{
		buzzer_flag = true;
		buzzer_counter = 0;
		digitalWrite(BUZZER, LOW);
	}

	// if power button is pressed, add 1 second to abort counter
	if (digitalRead(POWER_BUTTON) == LOW)
	{
		abort_counter++;
		user_presense_counter = 0;
		digitalWrite(LCD_LED, LOW);
	}
	// if power button is not pressed, zero abort counter
	else if (digitalRead(POWER_BUTTON) == HIGH)
	{
		abort_counter = 0;
	}
	// if power button is pressed for 5 consecutive seconds, initialize device
	if (abort_counter > 4)
	{
		device_status = 4;
		device_init();
#ifdef NETWORK
		device_status_change = 2;
#endif
	}
	
	// counter that manages 1/3 duty cycle for heater
	heater_counter++;
	if (heater_counter == 60)
	{
		heater_counter = 0;
	}

	if (device_status == 1 || device_status == 2)
	{
		// as long as device is busy, abstract 1 second from countdown
		if (countdown > 0)
		{
			countdown--;
			display_routine();
		}
		// if countdown is zero and device is busy, initialize device
		else if (countdown <= 0)
		{
			device_status = 3; // task finished, no commands accepted
			device_init();
#ifdef NETWORK
			device_status_change = 2;
			connect_flag = false;
#endif
		}
	}
	
	// buzz 5" before finish
	if (countdown == 4)
	{
		buzzer_flag = false;
	}

	if (user_presense_counter <= MAX_USER_PRESENSE_INTERVAL)
	{
		user_presense_counter++;
	}
	// if MAX_LCD_LED_TIME seconds have passed since last user input, turn LCD LED off
	if (user_presense_counter > MAX_LCD_LED_TIME)
	{
		digitalWrite(LCD_LED, HIGH);
	}

	// if device is busy, return start_command to zero and run runtime_routines function
	if (device_status == 1 || device_status == 2)
	{
#ifdef NETWORK
		start_command = 0;
#endif
		runtime_routines();
	}
#ifdef NETWORK
	else
	{
		stop_command = 0;
	}
#endif
}

void initialize_pin_IOs()
{
	pinMode(ANALOG_BUTTONS, INPUT);
	pinMode(THERMISTOR, INPUT);
	pinMode(MOTOR, OUTPUT); // (HIGH: motor on, LOW: motor off)
	pinMode(HEATER, OUTPUT); // (HIGH: heating element on, LOW: heating element off)
	pinMode(LCD_LED, OUTPUT); // (HIGH: LCD LED off, LOW: LCD LED on)
	pinMode(POWER_BUTTON, INPUT_PULLUP); // (HIGH: Motor on, LOW: motor off)
	// keep motor and heating element off, LCD LED on while booting
	digitalWrite(MOTOR, LOW);
	digitalWrite(HEATER, LOW);
	digitalWrite(LCD_LED, LOW);
}

void initialize_LCD()
{
	lcd.init();
	lcd.clear();
	lcd.gotoXY(0, 0);
	lcd.string(strings_lcd_init[0]);
	lcd.gotoXY(0, 1);
	lcd.string(strings_lcd_init[1]);
	lcd.gotoXY(0, 2);
	lcd.string(strings_lcd_init[2]);
	lcd.gotoXY(18, 3);
	lcd.string(strings_lcd_init[3]);
	lcd.gotoXY(0, 5);
	lcd.string(strings_lcd_init[4]);
}

void initialize_Timer1()
{
	cli(); // disable global interrupts
	TCCR1A = 0; // set entire TCCR1A register to 0
	TCCR1B = 0; // same for TCCR1B
	OCR1A = 15624; // set compare match register to desired timer count (1s)
	TCCR1B |= (1 << WGM12); // turn on CTC mode
	// Set CS10 and CS12 bits for 1024 prescaler:
	TCCR1B |= (1 << CS10);
	TCCR1B |= (1 << CS12);
	TIMSK1 |= (1 << OCIE1A); // enable timer compare interrupt
	sei(); // enable global interrupts
}

void setup()
{
#ifdef SERIAL_DEBUG
	Serial.begin(9600); // start serial communication at 9600 baud
	Serial.println();
	Serial.println(strings_serial_device_feedback[0]);
#endif
	initialize_pin_IOs();
	initialize_LCD();
#ifdef NETWORK
#ifdef SERIAL_DEBUG
	Serial.println(serial_net_feedback[0]);
#endif
	lcd.gotoXY(0, 4);
	lcd.string(strings_lcd_net_status[2]);
	if(Ethernet.begin(mac) > 0) // start ethernet client using MAC address and DHCP, if OK continue
	{
		network_status = 2;
		connection_status = true;
#ifdef SERIAL_DEBUG
		// send network info to serial
		Serial.print(serial_net_feedback[2]);
		Serial.println(Ethernet.localIP());
		Serial.print(serial_net_feedback[3]);
		Serial.println(Ethernet.subnetMask());
		Serial.print(serial_net_feedback[4]);
		Serial.println(Ethernet.gatewayIP());
		Serial.print(serial_net_feedback[5]);
		Serial.println(Ethernet.dnsServerIP());
#endif
	}
	// if ethernet client could not start
	else
	{
#ifdef SERIAL_DEBUG
		Serial.println(serial_net_feedback[1]);
#endif
		connection_status = false;
		network_status = 0;
		display_routine();
	}
#endif

	device_init();
	initialize_Timer1();

#ifdef SERIAL_DEBUG
	// send initialization feedback to serial
	serial_feedback();
	Serial.println(strings_serial_device_feedback[1]); // send "Device Ready." message to serial
	Serial.println();
#endif
}

#ifdef NETWORK
// connect to remote server and fetch commands
void connect_and_fetch()
{
	connect_flag = true; // prevent connect_and_fetch from running again
	if (client.connect(serverName[0], PORT) > 0) // if connection to remote server (port 80) is successful
	{
		// send device parameters and values to remote server via URL
		client.print(client_print_values[0]);
		client.print(menu);
		client.print(client_print_values[1]);
		client.print(loafsize);
		client.print(client_print_values[2]);
		client.print(browning);
		client.print(client_print_values[3]);
		client.print(device_status);
		client.print(client_print_values[4]);
		client.print(countdown);
		client.print(client_print_values[5]);
		client.print(digitalRead(MOTOR));
		client.print(client_print_values[6]);
		client.print(digitalRead(HEATER));
		client.print(client_print_values[7]);
		client.print(device_id);
		client.print(client_print_values[8]);
		client.print(device_hash);
		client.print(client_print_values[9]);
		if (device_status_change_flag == false) // if device_status_change_flag is false, send actual status change, else send 0
		{
			client.print(device_status_change);;
		}
		else
		{
			client.print("0");
		}
		client.println(client_print_values[10]);
		client.println();
		connection_status = true;
		network_status = 1;
		if (device_status != 1 && device_status != 2)
		{
			display_routine();
		}
		fetch_retries_counter = 0;
	}
	// if connection to remote server (port PORT) is unsuccessful
	else
	{
#ifdef SERIAL_DEBUG
		Serial.println(serial_net_feedback[6]);
#endif
		fetch_retries_counter++;
		reconnect_timer += FETCH_INTERVAL;
		if (fetch_retries_counter >= MAX_FETCH_RETRIES)
		{
			fetch_retries_counter = 0;
			connection_status = false;
			network_status = 0;
			if (device_status != 1 && device_status != 2)
			{
				display_routine();
			}
		}
	}
	// if ethernet client is connected to remote server
	if (client.connected())
	{
		if(client.find(ok_pattern[0])) // search for success pattern in HTML code
		{
			menu_command = client.parseInt(); // fetch menu command
			loafsize_command = client.parseInt(); // fetch loafsize command
			browning_command = client.parseInt(); // fetch browning command
			countdown_command = (unsigned long) client.parseFloat(); // fetch countdown command
			start_command = client.parseInt(); // fetch start command
			stop_command = client.parseInt(); // fetch stop command

			// if all commands are 0 credentials are invalid
			if (menu_command == 0 && loafsize_command == 0 && browning_command == 0 && countdown_command == 0)
			{
				network_status = 3;
				connection_status = false;
				lcd.gotoXY(0, 4);
				lcd.string(strings_lcd_net_status[3]);
#ifdef SERIAL_DEBUG
				Serial.println(serial_net_feedback[16]);
#endif
			}
			// if menu command is 403, account is banned
			else if (menu_command == 403)
			{
				network_status = 4;
				connection_status = false;
				lcd.gotoXY(0, 4);
				lcd.string(strings_lcd_net_status[4]);
#ifdef SERIAL_DEBUG
				Serial.println(serial_net_feedback[17]);
#endif
			}
			else
			{
#ifdef SERIAL_DEBUG
				// send fetched commands to serial
				Serial.print(serial_net_feedback[12]);
				Serial.println(menu_command);
				Serial.print(serial_net_feedback[13]);
				Serial.println(loafsize_command);
				Serial.print(serial_net_feedback[14]);
				Serial.println(browning_command);
				Serial.print(serial_net_feedback[15]);
				Serial.println(countdown_command);
				Serial.print(serial_net_feedback[8]);
				Serial.println(start_command);
				Serial.print(serial_net_feedback[9]);
				Serial.println(stop_command);
				Serial.println();
#endif
				if (device_status_change_flag == false)
				{
					device_status_change = 0;
					device_status_change_flag = true;
				}
				if (device_status_change == 1 || device_status_change == 2)
				{
					device_status_change_flag = false;
				}
				if ((device_status == 0 || device_status == 3 || device_status == 4) && start_command == 1) // if remote start command is given and device is idle or previous task aborted, pass commands to device parameters
				{
					menu = menu_command; //parse menu command from server value to system
					menu_routine();
					if (menu == 1 || menu == 2 || menu == 3 || menu == 5 || menu == 11) //parse loafsize command from server value to system
					{
						loafsize = loafsize_command;
						loafsize_routine();
					}
					if (menu >= 1 && menu <= 7) //parse browning and temperature command from server value to system
					{
						browning = browning_command;
						browning_temp();
					}
					countdown = countdown_command; //parse countdown command from server value to system
					device_status = 2; // set device status to remote control
					device_status_change = 1;
					buzzer_flag = false;
#ifdef SERIAL_DEBUG
					Serial.println(serial_net_feedback[10]);
					serial_feedback();
#endif
				}
				else if((device_status == 1 || device_status == 2) && stop_command == 1) // if remote abort command is given, report task aborted
				{
					device_status = 4;
					device_init();
					device_status_change = 2;
					connect_flag = false;

#ifdef SERIAL_DEBUG
					Serial.println(serial_net_feedback[11]);
#endif
				}
			}
		}
		client.stop();
	}
	else
	{
		client.stop();
	}
}

// reconnect to remote server if connection is down
void reconnect()
{
	network_status = 2;
	if (device_status != 1 && device_status != 2)
	{
		display_routine();
	}
	reconnect_flag = true; // prevent reconnect from running again
#ifdef SERIAL_DEBUG
	Serial.println(serial_net_feedback[7]);
#endif
	if(Ethernet.begin(mac) > 0) // restart ethernet client using MAC address and DHCP, if OK continue
	{
		connection_status = true;
#ifdef SERIAL_DEBUG
		// send network info to serial
		Serial.print(serial_net_feedback[2]);
		Serial.println(Ethernet.localIP());
		Serial.print(serial_net_feedback[3]);
		Serial.println(Ethernet.subnetMask());
		Serial.print(serial_net_feedback[4]);
		Serial.println(Ethernet.gatewayIP());
		Serial.print(serial_net_feedback[5]);
		Serial.println(Ethernet.dnsServerIP());
#endif
	}
	// if ethernet client could not start
	else
	{
		connection_status = false;
		network_status = 0;
		if (device_status != 1 && device_status != 2)
		{
			display_routine();
		}
#ifdef SERIAL_DEBUG
		Serial.println(serial_net_feedback[1]);
#endif
	}
}
#endif

void loop ()
{
	// while device status is idle or just finished task, wait for input from buttons, keep motor and heater off
	if (device_status != 1 && device_status != 2)
	{
		motor_off_heater_off();
		analog_buttons();
		power_button();
	}

	// while device status is busy, read input from buttons
	if (device_status == 1 || device_status == 2)
	{
		runtime_buttons();
	}

#ifdef NETWORK
	if (connect_flag == false)
	{
		connect_and_fetch();
	}
	if (reconnect_flag == false)
	{
		// while device status is idle or just finished task or task aborted, only reconnect if no user input has been given for MAX_USER_PRESENSE_INTERVAL seconds
		if ((device_status != 1 && device_status != 2) && user_presense_counter > MAX_USER_PRESENSE_INTERVAL)
		{
			reconnect();
		}
		else if (device_status == 1 || device_status == 2)
		{
			reconnect();
		}
	}
#endif
}
